@@ -4,7 +4,8 @@ from abc import abstractmethod
 from enum import Enum
 from typing import Optional, Tuple, Any, Iterator, IO
 
-from hiro_graph_client.client import Graphit, TokenHandler, APIConfig
+from hiro_graph_client.client import HiroGraph
+from hiro_graph_client.clientlib import TokenHandler, APIConfig
 from requests.exceptions import RequestException
 
 
@@ -28,7 +29,11 @@ class Entity(Enum):
     ATTACHMENT = "attachment"
 
 
-class ResultCallback:
+class HiroResultCallback:
+    """
+    Abstract class for objects that receive each result of a command in its method *result(...)*.
+    Objects of this type are given to the *HiroGraphBatch* as parameter *callback=*.
+    """
 
     @abstractmethod
     def result(self, data: Any, code: int) -> None:
@@ -37,19 +42,20 @@ class ResultCallback:
 
 class HiroConnection:
     """
-    Contains the Graphit Client and a possibly predefined token.
+    Contains the HiroGraph Client and a possibly predefined token.
     """
 
     token: str = None
     """Set a predefined token for the hiro connection."""
-    client: Graphit
-    """The python client for REST API"""
 
-    def __init__(self, client: Graphit, token: str = None):
+    client: HiroGraph
+    """The python client for HIRO REST Graph API"""
+
+    def __init__(self, client: HiroGraph, token: str = None):
         """
-        Contains the Graphit Client and a possibly predefined token.
+        Contains the HiroGraph Client and a possibly predefined token.
 
-        :param client: Required: Graphit Client
+        :param client: Required: HiroGraph Client
         :param token: Optional: Predefined token
         """
         self.client = client
@@ -251,7 +257,7 @@ class HiroBatchRunner:
 
     def get_id_by_xid(self, ogit_xid: str) -> Optional[str]:
         """
-        Get an ogit/_id from the *xid_cache* or get it from Graphit when it is not in the cache is or it is
+        Get an ogit/_id from the *xid_cache* or get it from HiroGraph when it is not in the cache is or it is
         disabled.
 
         :param ogit_xid: The xid to use to look for an ogit/_id.
@@ -274,7 +280,7 @@ class HiroBatchRunner:
 
     def check_id(self, ogit_id: str) -> Optional[str]:
         """
-        Check for existence of an id in *xid_cache* or, if not found there or cache is disabled, in Graphit.
+        Check for existence of an id in *xid_cache* or, if not found there or cache is disabled, in HiroGraph.
 
         :param ogit_id: The ogit/_id to check.
         :return: The ogit/_id of the result or None
@@ -293,7 +299,7 @@ class HiroBatchRunner:
         Get the ogit/_id from the source data given in *attributes*.
 
         * If the ogit/_id is present in *attributes*, return it.
-        * If ogit/_id is missing but ogit/_xid is present, use the ogit/_xid to get the ogit/_id from Graphit and
+        * If ogit/_id is missing but ogit/_xid is present, use the ogit/_xid to get the ogit/_id from HiroGraph and
           return it.
 
         :param attributes: Dict of attributes.
@@ -320,7 +326,7 @@ class HiroBatchRunner:
         Get the ogit/_id from the source data given in *attributes*.
 
         * If the ogit/_id is present in *attributes*, return it.
-        * If ogit/_id is missing but ogit/_xid is present, use the ogit/_xid to get the ogit/_id from Graphit and
+        * If ogit/_id is missing but ogit/_xid is present, use the ogit/_xid to get the ogit/_id from HiroGraph and
           return it.
 
         :param attributes: Dict of attributes.
@@ -388,7 +394,7 @@ class HiroBatchRunner:
         To be used with self.for_each_attribute()
 
         Try to resolve_ids keys that start with "id:" or "xid:". Try to find the
-        ogit/_id of a vertex by using the value for such a key in graphit. Return a tuple of (key, value) with the key
+        ogit/_id of a vertex by using the value for such a key in the graph. Return a tuple of (key, value) with the key
         without its prefix "id:" or "xid:" and the value resolved to a real "ogit/_id".
 
         :param key: Attribute key
@@ -554,7 +560,7 @@ class CreateVerticesRunner(HiroBatchRunner):
         Create vertices
 
         :param session_data: Required: Session data / caches.
-        :param connection: Required: The handler for the connection to HIRO Graphit.
+        :param connection: Required: The handler for the connection to HIRO HiroGraph.
         """
         super().__init__(Entity.VERTEX, Action.CREATE, session_data, connection)
 
@@ -584,7 +590,7 @@ class UpdateVerticesRunner(HiroBatchRunner):
         Update vertices
 
         :param session_data: Required: Session data / caches.
-        :param connection: Required: The handler for the connection to HIRO Graphit.
+        :param connection: Required: The handler for the connection to HIRO HiroGraph.
         """
         super().__init__(Entity.VERTEX, Action.UPDATE, session_data, connection)
 
@@ -613,7 +619,7 @@ class DeleteVerticesRunner(HiroBatchRunner):
         Delete vertices
 
         :param session_data: Required: Session data / caches.
-        :param connection: Required: The handler for the connection to HIRO Graphit.
+        :param connection: Required: The handler for the connection to HIRO HiroGraph.
         """
         super().__init__(Entity.VERTEX, Action.DELETE, session_data, connection)
 
@@ -641,7 +647,7 @@ class HandleVerticesRunner(HiroBatchRunner):
         Handle vertices. Either update or create them based on incoming payload entries.
 
         :param session_data: Required: Session data / caches.
-        :param connection: Required: The handler for the connection to HIRO Graphit.
+        :param connection: Required: The handler for the connection to HIRO HiroGraph.
         """
         super().__init__(Entity.VERTEX, Action.UNDEFINED, session_data, connection)
 
@@ -682,7 +688,7 @@ class CreateEdgesRunner(HiroBatchRunner):
         Create edges between vertices
 
         :param session_data: Required: Session data / caches.
-        :param connection: Required: The handler for the connection to HIRO Graphit.
+        :param connection: Required: The handler for the connection to HIRO HiroGraph.
         """
         super().__init__(Entity.EDGE, Action.CREATE, session_data, connection)
 
@@ -708,7 +714,7 @@ class DeleteEdgesRunner(HiroBatchRunner):
         Delete edges between vertices
 
         :param session_data: Required: Session data / caches.
-        :param connection: Required: The handler for the connection to HIRO Graphit.
+        :param connection: Required: The handler for the connection to HIRO HiroGraph.
         """
         super().__init__(Entity.EDGE, Action.DELETE, session_data, connection)
 
@@ -734,7 +740,7 @@ class AddTimeseriesRunner(HiroBatchRunner):
         Attach timeseries values to a vertex.
 
         :param session_data: Required: Session data / caches.
-        :param connection: Required: The handler for the connection to HIRO Graphit.
+        :param connection: Required: The handler for the connection to HIRO HiroGraph.
         """
         super().__init__(Entity.TIMESERIES, Action.CREATE, session_data, connection)
 
@@ -759,7 +765,7 @@ class AddAttachmentRunner(HiroBatchRunner):
         Attach an attachment to a vertex.
 
         :param session_data: Required: Session data / caches.
-        :param connection: Required: The handler for the connection to HIRO Graphit.
+        :param connection: Required: The handler for the connection to HIRO HiroGraph.
         """
         super().__init__(Entity.ATTACHMENT, Action.CREATE, session_data, connection)
 
@@ -790,9 +796,9 @@ class AddAttachmentRunner(HiroBatchRunner):
             raise SourceValueError('"data" not found or empty in "attributes._content_data".')
 
 
-class GraphitBatch:
+class HiroGraphBatch:
     """
-    This class handles lists of vertex-, edge- or timeseries-data via Graphit.
+    This class handles lists of vertex-, edge- or timeseries-data via HiroGraph.
     """
 
     api_config: APIConfig
@@ -800,7 +806,7 @@ class GraphitBatch:
     request_queue: queue.Queue
     result_queue: queue.Queue
 
-    callback: ResultCallback
+    callback: HiroResultCallback
 
     use_xid_cache: bool
     """Use xid caching. Default is True when omitted or set to None."""
@@ -816,18 +822,17 @@ class GraphitBatch:
         "add_timeseries",
         "add_attachments"
     ]
-    """This is the list of commands (method names) that GraphitBatch handles."""
+    """This is the list of commands (method names) that HiroGraphBatch handles."""
 
     def __init__(self,
                  graph_endpoint: str,
-                 callback: ResultCallback,
+                 callback: HiroResultCallback = None,
                  hiro_token: str = None,
                  username: str = None,
                  password: str = None,
                  client_id: str = None,
                  client_secret: str = None,
                  auth_endpoint: str = None,
-                 iam_endpoint: str = None,
                  use_xid_cache: bool = True,
                  proxies: dict = None,
                  parallel_workers: int = 8,
@@ -835,19 +840,18 @@ class GraphitBatch:
         """
         Constructor
 
-        Use the connection to HIRO Graphit either by giving a predefined *hiro_token* or by
+        Use the connection to HIRO HiroGraph either by giving a predefined *hiro_token* or by
         specifying all other parameters needed for authentication.
 
         :param graph_endpoint: required: URL of the graph.
-        :param callback: required: Callback object for results.
+        :param callback: required when multi_command() is used, optional otherwise: Callback object for results.
         :param hiro_token: optional, required if other authentication data is missing: Predefined token to authenticate
                            against HIRO. Overrides all other authentication data.
-        :param username: optional, required if *hiro_token* is None: Username for Graphit .
-        :param password: optional, required if *hiro_token* is None: Password for Graphit.
+        :param username: optional, required if *hiro_token* is None: Username for HiroGraph .
+        :param password: optional, required if *hiro_token* is None: Password for HiroGraph.
         :param client_id: optional, required if *hiro_token* is None: Id for Authentication (OAuth2).
         :param client_secret: optional, required if *hiro_token* is None: Secret for Authentication (OAuth2).
         :param auth_endpoint: optional, required if *hiro_token* is None: URL of the authentication API.
-        :param iam_endpoint: optional: URL of the IAM instance for accessing accounts. Default is None.
         :param use_xid_cache: Use xid caching. Default is True when omitted or set to None.
         :param proxies: Proxy configuration for *requests*. Default is None.
         :param parallel_workers: Amount of parallel workers for requests. Default is 8.
@@ -880,9 +884,8 @@ class GraphitBatch:
                                     password=password,
                                     client_id=client_id,
                                     client_secret=client_secret,
-                                    graph_endpoint=graph_endpoint,
+                                    endpoint=graph_endpoint,
                                     auth_endpoint=auth_endpoint,
-                                    iam_endpoint=iam_endpoint,
                                     raise_exceptions=True,
                                     proxies=proxies)
 
@@ -920,7 +923,7 @@ class GraphitBatch:
         if not session:
             session = self.__init_session()
         if not connection:
-            connection = HiroConnection(Graphit.new_from(self.api_config, session.token_handler), self.hiro_token)
+            connection = HiroConnection(HiroGraph.new_from(self.api_config, session.token_handler), self.hiro_token)
         return session, connection
 
     def create_vertices(self, attributes: dict, connection: HiroConnection = None, session: SessionData = None):
@@ -938,7 +941,7 @@ class GraphitBatch:
         """
         Update vertex from *attributes*.
 
-        Attributes needs at least a column "ogit/_id" or "ogit/_xid" to find the vertex to be updated.
+        Attributes needs at least a key "ogit/_id" or "ogit/_xid" to find the vertex to be updated.
         It then sanitizes the payload by ignoring every attribute starting with "ogit/_" (unless "ogit/_owner",
         "ogit/_content" or "ogit/_tags") of the dict before attempting to update.
 
@@ -953,7 +956,7 @@ class GraphitBatch:
         """
         Delete vertex given by *attributes*.
 
-        Attributes needs at least a column "ogit/_id" or "ogit/_xid" to find the vertex to be deleted.
+        Attributes needs at least a key "ogit/_id" or "ogit/_xid" to find the vertex to be deleted.
 
         :param attributes: Dict containing the attributes for the vertex.
         :param connection: optional: Connection to use. A new connection will be used if this is not set.
@@ -980,7 +983,7 @@ class GraphitBatch:
         """
         Create edge from *attributes*.
 
-        Attributes needs a column "from:ogit/_id" or "from:ogit/_xid" and
+        Attributes needs a key "from:ogit/_id" or "from:ogit/_xid" and
         "to:ogit/_id" or "to:ogit/_xid" as well as "verb" to be able to determine the vertices to connect.
 
         :param attributes: Dict containing the fields "from:...,verb,to:..." for the edge.
@@ -994,7 +997,7 @@ class GraphitBatch:
         """
         Delete edge given by *attributes*.
 
-        Attributes needs a column "from:ogit/_id" or "from:ogit/_xid" and
+        Attributes needs a key "from:ogit/_id" or "from:ogit/_xid" and
         "to:ogit/_id" or "to:ogit/_xid" as well as "verb" to be able to determine the edge between the vertices to
         delete.
 
@@ -1009,7 +1012,7 @@ class GraphitBatch:
         """
         Update vertices with timeseries data.
 
-        Attributes needs at least a column "ogit/_id" or "ogit/_xid" to find the vertex to be updated.
+        Attributes needs at least a key "ogit/_id" or "ogit/_xid" to find the vertex to be updated.
 
         :param attributes: Contains the timeseries items.
         :param connection: optional: Connection to use. A new connection will be used if this is not set.
@@ -1089,14 +1092,17 @@ class GraphitBatch:
 
             self.request_queue.put(('add_attachments', attributes))
 
-    def _reader(self) -> None:
+    def _reader(self, collected_results: list) -> None:
         """
         Thread executor function. Read items from the *self.result_queue* and call the callback function with them.
 
         Thread exits when *self.result_queue.get()* reads None.
         """
         for result, code in iter(self.result_queue.get, None):
-            self.callback.result(result, code)
+            if self.callback is not None:
+                self.callback.result(result, code)
+            if collected_results is not None:
+                collected_results.append(result)
             self.result_queue.task_done()
 
     def _worker(self, session: SessionData) -> None:
@@ -1108,7 +1114,7 @@ class GraphitBatch:
 
         :param session: The session object to share between all connections.
         """
-        connection = HiroConnection(Graphit.new_from(self.api_config, session.token_handler), self.hiro_token)
+        connection = HiroConnection(HiroGraph.new_from(self.api_config, session.token_handler), self.hiro_token)
 
         for command, attributes in iter(self.request_queue.get, None):
             func = getattr(self, command, None)
@@ -1116,30 +1122,55 @@ class GraphitBatch:
                 func(attributes, connection, session)
             self.request_queue.task_done()
 
-    def multi_command(self, command_iter: Iterator[dict]) -> None:
+    def multi_command(self, command_iter: Iterator[dict]) -> Optional[list]:
         """
         Run a multi-command batch.
 
-        The command_iter iterates over a dict with pairs
+        The command_iter iterates over a list of dicts with pairs like
 
         ::
 
             {
-                "[command]": payload,
-                "[command]": payload
-            }
+                "[command]": { "[key]": "[value]", ... }
+            },
+            {
+                "[command]": { "[key]": "[value]", ... }
+            },
+            ...
+
+        or
+
+        ::
+
+            {
+                "[command]": [
+                    { "[key]": "[value]", ... },
+                    { "[key]": "[value]", ... }
+                ]
+            },
+            ...
+
 
         with payload being a list of dict containing the attributes to run with that command.
 
         :param command_iter: An iterator for a dict of pairs "[command]:payload".
+        :return a list with results when no callback is set, None otherwise.
         """
         with concurrent.futures.ThreadPoolExecutor() as executor:
 
+            def _request_queue_put(_command: str, _attributes: dict) -> None:
+                if isinstance(_attributes, dict):
+                    self.request_queue.put((_command, _attributes))
+                else:
+                    raise SourceValueError("Found attributes that are not a dict.")
+
             session = self.__init_session()
 
-            executor.submit(GraphitBatch._reader, self)
+            collected_results = [] if self.callback is None else None
+
+            executor.submit(HiroGraphBatch._reader, self, collected_results)
             for _ in range(self.parallel_workers):
-                executor.submit(GraphitBatch._worker, self, session)
+                executor.submit(HiroGraphBatch._worker, self, session)
 
             handle_session_data = False
             for command_entry in command_iter:
@@ -1149,13 +1180,20 @@ class GraphitBatch:
                         command = "handle_vertices"
                         handle_session_data = True
 
-                    if command in self.commands:
-                        self.request_queue.put((command, attributes))
-                    else:
+                    try:
+                        if command in self.commands:
+                            if isinstance(attributes, list):
+                                for attribute_entry in attributes:
+                                    _request_queue_put(command, attribute_entry)
+                            else:
+                                _request_queue_put(command, attributes)
+                        else:
+                            raise SourceValueError("No such command \"{}\".".format(command))
+                    except SourceValueError as err:
                         sub_result, sub_code = HiroBatchRunner.error_message(
                             Entity.UNDEFINED,
                             Action.UNDEFINED,
-                            SourceValueError("No such command \"{}\".".format(command)),
+                            err,
                             attributes,
                             400), 400
 
@@ -1174,6 +1212,8 @@ class GraphitBatch:
 
             self.result_queue.put(None)
 
+            return collected_results
+
 
 class SourceValueError(ValueError):
     """
